/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.asgteach.familytree.eventsviewer;

import com.asgteach.familytree.capabilityinterfaces.ReloadableViewCapability;
import com.asgteach.familytree.model.FamilyTreeManager;
import com.asgteach.familytree.utilities.DeleteNodeAction;
import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ActionMap;
import org.netbeans.api.settings.ConvertAsProperties;
import org.openide.LifecycleManager;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.BeanTreeView;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle.Messages;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;

/**
 * Top component which displays something.
 */
@ConvertAsProperties(dtd = "-//org.asgteach.familytree.eventsviewer//Events//EN",
autostore = false)
@TopComponent.Description(preferredID = "EventsViewerTopComponent",
iconBase = "com/asgteach/familytree/utilities/eventIcon.png",
persistenceType = TopComponent.PERSISTENCE_ALWAYS)
@TopComponent.Registration(mode = "explorer", openAtStartup = true)
@ActionID(category = "Window", id = "com.asgteach.familytree.eventsviewer.EventsViewerTopComponent")
@ActionReference(path = "Menu/Window" /*
 * , position = 333
 */)
@TopComponent.OpenActionRegistration(displayName = "#CTL_EventsAction",
preferredID = "EventsViewerTopComponent")
@Messages({
    "CTL_EventsAction=EventsViewer",
    "CTL_EventsTopComponent=Events Viewer",
    "HINT_EventsTopComponent=This is an Events Viewer window"
})
public final class EventsViewerTopComponent extends TopComponent implements
        PropertyChangeListener, ExplorerManager.Provider, LookupListener {

    private final ExplorerManager em = new ExplorerManager();
    private final Lookup lookup;
    private FamilyTreeManager ftm;
    private Lookup.Result<Node> result;
    private static final Logger logger = Logger.getLogger(EventsViewerTopComponent.class.getName());

    public EventsViewerTopComponent() {
        initComponents();
        setName(Bundle.CTL_EventsTopComponent());
        setToolTipText(Bundle.HINT_EventsTopComponent());
        setLayout(new BorderLayout());
        BeanTreeView btv = new BeanTreeView();
        btv.setRootVisible(true);
        add(btv, BorderLayout.CENTER);

        ActionMap map = this.getActionMap();
        // associates the key 'delete' with the delete action
        map.put(DeleteNodeAction.ACTION_MAP_DELETE_KEY, ExplorerUtils.actionDelete(em, true));
        this.lookup = ExplorerUtils.createLookup(this.em, map);
        this.associateLookup(lookup);

        EventRootNode node = new EventRootNode();
        em.setRootContext(node);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    @Override
    public void componentOpened() {
        this.ftm = Lookup.getDefault().lookup(FamilyTreeManager.class);
        if (ftm == null) {
            logger.log(Level.SEVERE, "Cannot get FamilyTreeManager object");
            LifecycleManager.getDefault().exit();
        }
        ftm.addPropertyChangeListener(this);
        result = Utilities.actionsGlobalContext().lookupResult(Node.class);
        result.addLookupListener(this);
    }

    @Override
    public void componentClosed() {
        result.removeLookupListener(this);
        ftm.removePropertyChangeListener(this);
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        // TODO store your settings
    }

    void readProperties(java.util.Properties p) {
        String version = p.getProperty("version");
        // TODO read your settings according to their version
    }

    @Override
    public ExplorerManager getExplorerManager() {
        return em;
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        // Person can be destroyed by actions within another topcomponent
        // so we need to reload the view of a Person is destroyed
        if (pce.getPropertyName().equals(FamilyTreeManager.PROP_PERSON_DESTROYED)
                || pce.getPropertyName().equals(FamilyTreeManager.PROP_EVENT_ADDED)) {
            logger.log(Level.FINER, "Change Event from FTM");            
            reloadView();
        }
    }

    private void reloadView() {
        Node node = this.em.getRootContext();
        ReloadableViewCapability rvc =
                node.getLookup().lookup(ReloadableViewCapability.class);
        try {
            rvc.reloadChildren();
        } catch (Exception e) {
            Exceptions.printStackTrace(e);
        }
    }
    
    @Override
    public void resultChanged(LookupEvent ev) {
        Collection<? extends Node> nodes = result.allInstances();
        // only get the selection if there is one
        // otherwise leave the selection unchanged!
        
        // If you want the selection window to clear
        // when the user clicks on a non-node thingy,
        // then remove the test for isEmpty
        if (!nodes.isEmpty()) {
            logger.log(Level.FINER, "Detected selection change");  
            try {
                em.setSelectedNodes(nodes.toArray(new Node[nodes.size()]));                
            } catch (PropertyVetoException ex) {
                logger.log(Level.WARNING, null, ex);
            }
        }
    }
}
